Framework BRL.Basic
Import BRL.Retro
Import BRL.EventQueue
Import BRL.Stream
Import BRL.Threads
Import MaxGUI.Drivers

Type TMatch
	Field File:String
	Field Matches:Int[]
	
	Function Create:TMatch(File:String, Matches:Int[])
		Local Match:TMatch = New TMatch
		Match.File = File
		Match.Matches = Matches
		Return Match
	EndFunction
EndType

Function FindMatches:Int[](File:String, Text:String)
	Local Stream:TStream = ReadStream(File)
	Local Matches:Int[]
	If Stream Then
		Local LineID:Int = 0
		While Not Eof(Stream)
			LineID = LineID + 1
			Local Line:String = ReadLine(Stream)
			If Instr(Line, Text) Then
				Local MatchID = Matches.length
				Matches = Matches[.. MatchID + 1]
				Matches[MatchID] = LineID
			EndIf
		Wend
		CloseStream(Stream)
	EndIf
	Return Matches
EndFunction

Function LoopFind:TList(Folder:String, Text:String)
	Local Files:String[] = LoadDir(Folder)
	Local MatchList:TList = CreateList()
	FileCount = FileCount + Files.length
	
	For File:String = EachIn Files
		Local RealFile:String = Folder+"/"+File
		Select FileType(RealFile)
			Case 2
				Local MatchListB:TList = LoopFind(RealFile, Text)
				For Match:TMatch = EachIn MatchListB
					ListAddLast MatchList, Match
				Next
			Case 1
				Local Matches:Int[] = FindMatches(RealFile, Text)
				If Matches.length > 0 Then
					ListAddLast MatchList, TMatch.Create(RealFile, Matches)
				EndIf
		EndSelect
		
		FileCounted = FileCounted + 1
		Local Percent:Int = Int(Int(FileCounted*100)/FileCount)
		UpdateProgBar(Progress, Float(Percent)/100)
		SetStatusText Window, Percent+"% - "+RealFile
	Next
	Return MatchList
EndFunction

Function Find:Object(Data:Object)
	FileCount = 0
	FileCounted = 0
	
	Local Path:String = TextFieldText(TextField)
	Local MatchList:TList = LoopFind(TextFieldText(FolderField), Path)
	SetStatusText Window, "Finished"
	If MatchList.isEmpty() Then
		Notify "Couldn't match text in any file", True
	Else
		Local MatchesNotification:String
		For Match:TMatch = EachIn MatchList
			MatchesNotification = MatchesNotification + "In file "+Mid(Match.File, Len(Path))+", on line(s): "
			For I:Int = 0 To Match.Matches.length - 1
				Local Line:String = Match.Matches[I]
				MatchesNotification = MatchesNotification + Line + ","
			Next
			MatchesNotification = Mid(MatchesNotification, 1, Len(MatchesNotification) - 1) + "~n"
		Next
		SetTextAreaText FindsArea, MatchesNotification
		ShowGadget FindsWindow
	EndIf
EndFunction

Global FileCount:Int
Global FileCounted:Int

AppTitle = "Advanced Text Finder"
Global Window:TGadget = CreateWindow("Advanced Text Finder", 0, 0, 400, 210, Null, WINDOW_TITLEBAR|WINDOW_CENTER|WINDOW_MENU|WINDOW_STATUS)
Global HelpMenu:TGadget = CreateMenu("Help", 0, Window)
Global AboutMenu:TGadget = CreateMenu("About", 2, HelpMenu)
UpdateWindowMenu(Window)

Global FindsWindow:TGadget = CreateWindow("Found in files", 0, 0, 700, 230, Window, WINDOW_TITLEBAR|WINDOW_CENTER|WINDOW_HIDDEN)
Global FindsArea:TGadget = CreateTextArea(10, 10, 680, 180, FindsWindow, TEXTAREA_READONLY)

Global Progress:TGadget = CreateProgBar(120, 110, 190, 20, Window)
Global TextField:TGadget = CreateTextField(10, 35, 300, 20, Window)
Global FolderField:TGadget = CreateTextField(10, 85, 300, 20, Window)
Global SearchButton:TGadget = CreateButton("...", 320, 85, 50, 20, Window)
Global FindButton:TGadget = CreateButton("Find", 10, 110, 100, 20, Window)

Global FindThread:TThread

CreateLabel("Text to find", 10, 15, 100, 20, Window)
CreateLabel("Folder to find in", 10, 65, 100, 20, Window)

Repeat
	WaitEvent()
	Local Gadget:TGadget = TGadget(EventSource())
	Local Path:String
	
	Select EventID()
		Case EVENT_GADGETACTION
			Gadget = TGadget(EventSource())
			Select Gadget
				Case SearchButton
					Path= RequestDir("")
					SetGadgetText FolderField, Path
				Case FindButton
					If FindThread And ThreadRunning(FindThread) Then
						Notify "Wait until the current process is finished"
					Else
						FindThread = CreateThread(Find, Null)
					EndIf
			EndSelect
		Case EVENT_WINDOWCLOSE
			Gadget = TGadget(EventSource())
			Select Gadget
				Case Window
					End
				Case FindsWindow
					HideGadget(FindsWindow)
			EndSelect
		Case EVENT_MENUACTION
			Select Gadget
				Case AboutMenu
					Notify "Developed by Starkkz~nContact: http://www.unrealsoftware.de/profile.php?userid=10464"
			EndSelect
	EndSelect
Forever